#!/usr/bin/env python
#
# PLEASE REPORT BUGS, QUESTIONS AND COMMENTS TO JAN.PERTHOLD@BOKU.AC.AT
#

#################################
#                               #
#         GroScore 0.71a        #
#    jan.perthold@boku.ac.at    #
#                               #
#################################

import string, math, array
import os, sys, glob, re, time, argparse
import numpy as np

#------------------------------------------------------

parser = argparse.ArgumentParser(description="Input files for GroScore")
parser.add_argument('-pp','--pullparams', type=str, default="pp.gscore", required=True, help="GroScore pulling parameter file")
parser.add_argument('-sp','--structparams', type=str, default="sp.gscore", required=True, help="GroScore strucutre parameter file")
args=parser.parse_args()

#------------------------------------------------------

def readpullparams(filepath):
  params = []
  if os.path.isfile(filepath):
    with open(filepath, "rb") as f:
      for line in f:
        if not line.strip().startswith("#"):
          tmp = line.split()
          for i in tmp:
            try:
              params.append(float(i))
            except (ValueError, IndexError, AttributeError):
              j = 0 
  return params

#------------------------------------------------------

def readstructparams(filepath):
  num = []
  start = []
  if os.path.isfile(filepath):
    with open(filepath, "rb") as f:
      for line in f:
        if not line.strip().startswith("#"):
          tmp = line.split()
          try:
            num.append(int(tmp[0]))
            start.append(int(tmp[1]))
          except (ValueError, IndexError, AttributeError):
            i = 0
  if len(num) == len(start):
    params = np.zeros(shape=(len(num),2))
    i = 0
    while i < len(num):
      params[i,0] = num[i]
      params[i,1] = start[i]
      i += 1
    return params
  else:
    return False

#------------------------------------------------------

def readtwocolumns(filepath):
  num = []
  num2 = []
  if os.path.isfile(filepath):
    with open(filepath, "rb") as f:
      for line in f:
        if not line.strip().startswith("#"):
          tmp = line.split()
          try:
            num.append(int(tmp[0]))
            num2.append(tmp[1])
          except (ValueError, IndexError, AttributeError):
            i = 0
  if len(num) == len(num2):
    return num, num2
  else:
    return False

#------------------------------------------------------

def readtwocolumnsfloat(filepath):
  num = []
  num2 = []
  if os.path.isfile(filepath):
    with open(filepath, "rb") as f:
      for line in f:
        if not line.strip().startswith("#"):
          tmp = line.split()
          try:
            num.append(int(tmp[0]))
            num2.append(float(tmp[1]))
          except (ValueError, IndexError, AttributeError):
            i = 0
  if len(num) == len(num2):
    return num, num2
  else:
    return False

#------------------------------------------------------

def countlines(filepath):
  i = 0
  if os.path.isfile(filepath):
    with open(filepath, "rb") as f:
      for line in f:
        if not line.strip().startswith("#"):
          i += 1
  return i

#------------------------------------------------------

print ""
print "#################################"
print "#                               #"
print "#         GroScore 0.71a        #"
print "#    jan.perthold@boku.ac.at    #"
print "#                               #"
print "#################################"
print ""

print "Reading input parameters........................."
pullparams = readpullparams(args.pullparams)
structparams = readstructparams(args.structparams)
calcstruct = np.zeros(shape=(structparams.shape[0],len(pullparams)+1))
calcstruct[:,0] = 1.0
frenstruct = np.zeros(shape=(structparams.shape[0],len(pullparams)))
print "Done."
print "GroScore will calculate a binding free energy estimate for " + str(structparams.shape[0]) + " structures."
print ""

j = 0
while j <= len(pullparams):

  #STAGE 0 AND FILE OF STAGE 0 DOES NOT EXIST
  if j == 0 and not os.path.isfile("results_%.0f.gscore"%j):
    print "-------------------------------------------------"
    print "Stage 0 - Initial Checks and Setup"
    print "-------------------------------------------------"
    i = 0
    while i < structparams.shape[0]:
      if os.path.exists("./%.0f"%structparams[i,0]):
        f = open("./%.0f/run.gscore"%structparams[i,0], "a")
        f.write("0 %.0f %.0f\n"%(structparams[i,0],structparams[i,1]))
        f.close()
        os.system("cp job.gscore ./%.0f/; cp submithandler.gscore ./%.0f/; cd ./%.0f; ./submithandler.gscore"%(structparams[i,0],structparams[i,0],structparams[i,0]))
      else:
        print "Structure %.0f: directory doesn't exist."%structparams[i,0]
        f = open("results_0.gscore", "a")
        f.write("%.0f NODIR\n"%structparams[i,0])
        f.close()
      i += 1
    #wait until this stage has finsihed
    print "Stage %.0f: waiting to finish."%j
    i = -1
    while i != np.sum(calcstruct[:,j],axis=0):
      i = countlines("results_%.0f.gscore"%(j))
      time.sleep(2)

  #FILE OF THIS STAGE DOES NOT EXIST
  if j != 0 and not os.path.isfile("results_%.0f.gscore"%j):
    print "-------------------------------------------------"
    print "Stage %.0f - Pulling of TOP %.3f%%."%(j,100.0*pullparams[j-1])
    print "-------------------------------------------------"
    if j == 1 and pullparams[j-1] != 1.0:
      print "ERROR: First fraction is not 100% - don't know what to do!"
      break
    if pullparams[j-1] > 1.0:
      print "ERROR: Fraction is > 100% - don't know what to do!"
      break
    #submit all the structures that should be calculated
    i = 0
    while i < structparams.shape[0]:
      if os.path.exists("./%.0f"%structparams[i,0]) and calcstruct[i,j] == 1:
        f = open("./%.0f/run.gscore"%structparams[i,0], "a")
        f.write("%.0f %.0f %.0f\n"%(j,structparams[i,0],structparams[i,1]))
        f.close()
        os.system("cp job.gscore ./%.0f/; cp submithandler.gscore ./%.0f/; cd ./%.0f; ./submithandler.gscore"%(structparams[i,0],structparams[i,0],structparams[i,0]))
      i += 1
    #wait until this stage has finsihed
    print "Stage %.0f: waiting to finish."%j
    i = -1
    while i != np.sum(calcstruct[:,j],axis=0):
      i = countlines("results_%.0f.gscore"%(j))
      time.sleep(2)

  #FILE OF THIS STAGE EXISTS
  if os.path.isfile("results_%.0f.gscore"%j):
    print "Stage %.0f: waiting to finish."%j
    #wait until this stage has finsihed
    i = -1
    while i != np.sum(calcstruct[:,j],axis=0):
      i = countlines("results_%.0f.gscore"%(j))
      time.sleep(2)
    print "Stage %.0f: finished."%j
    #STAGE 0
    if j == 0:
      results1, results2 = readtwocolumns("results_%.0f.gscore"%(j))
      i = 0
      while i < len(results1):
        if results2[i] == "OK":
          #if structure is ok, allow calculation in stage 1
          k = 0
          while k < calcstruct.shape[0]:
            if structparams[k,0] == results1[i]:
              calcstruct[k,j+1] = 1
            k += 1
        i += 1
      np.savetxt("calcstruct.gscore",calcstruct,delimiter="\t")
    #STAGE > 0
    else:
      fren = np.zeros(shape=(structparams.shape[0],2))
      #read in all stages until this stage j
      i = 1
      while i <= j:
        results1, results2 = readtwocolumnsfloat("results_%.0f.gscore"%(j))
        #load results into frenstruct and mark as calulated in calcstruct
        k = 0
        while k < len(results1):
          l = 0
          while l < calcstruct.shape[0]:
            if structparams[l,0] == results1[k]:
              frenstruct[l,j-1] = results2[k]
              calcstruct[l,j] = 1
            l += 1
          k += 1
        i += 1
      #now do ranking until this stage j
      fren[:,0] = structparams[:,0]
      i = 0
      while i < calcstruct.shape[0]:
        k = 0
        first = 1
        lnexpave = 0
        while k < frenstruct.shape[1]:
          if calcstruct[i,k+1] == 1.0 and first == 1:
            lnexpave = frenstruct[i,k] / 2.57734
            first = 0
          if calcstruct[i,k+1] == 1.0 and first == 0:
            exp = frenstruct[i,k] / 2.57734
            lnexpave = max(lnexpave, exp) + math.log(1.0 + math.exp(min(lnexpave, exp) - max(lnexpave, exp)))
          k += 1
        fren[i,1] = 2.57734 * lnexpave
        i += 1
      fren = fren[fren[:,1].argsort()]
      np.savetxt("fren.gscore",fren,delimiter="\t")
      np.savetxt("frenstruct.gscore",frenstruct,delimiter="\t")
      # for next stage, take top x percent
      i = 0
      while i < calcstruct.shape[0]*pullparams[j]:
        k = 0
        while k < calcstruct.shape[0]:
          if fren[i,0] == structparams[k,0] and calcstruct[k,1] == 1:
            calcstruct[k,j+1] = 1
          k += 1
        i += 1
      np.savetxt("calcstruct.gscore",calcstruct,delimiter="\t")
  j += 1




#while i 
