#!/usr/bin/env python
#
# PLEASE REPORT BUGS, QUESTIONS AND COMMENTS TO JAN.PERTHOLD@BOKU.AC.AT
#

import string
import math
import array
import os, sys, glob, re, time, argparse
import numpy as np

#------------------------------------------------------

parser = argparse.ArgumentParser(description="Calulate necesaary rotations to align CA-COMS with the z-axis.")
parser.add_argument('-f','--confile', type=str, default="conf.gro", required=True, help="GROMACS coordinate file.")
parser.add_argument('-s','--startb', type=int, default="24", required=True, help="Start residue for protein B.")
parser.add_argument('-br','--boxr', type=float, default="10", required=True, help="Simulation box diameter.")
args=parser.parse_args()

#------------------------------------------------------

def getdistance(x1, y1, z1, x2, y2, z2):
  """ Gives the distance between two points in 3D space.
  """  
  return math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2))

def getCOG(x, y, z):
  """ Gives the COG.
  """  
  return np.average(x), np.average(y), np.average(z)

#------------------------------------------------------

# read coordinate file - only CAs
num = []
num2 = []
xC = []
xC2 = []
yC = []
yC2 = []
zC = []
zC2 = []
if os.path.isfile(args.confile):
  with open(args.confile, "rb") as f:
    for line in f:
      if not line.strip().startswith("#"):
        tmp = line.split()
        try:
          s = re.search(r"\d+(\.\d+)?", tmp[0])
          tmp[0] = s.group(0)
          if int(tmp[0]) < args.startb and tmp[1] == "CA":
            num.append(int(tmp[2]))
            xC.append(float(tmp[3]))
            yC.append(float(tmp[4]))
            zC.append(float(tmp[5]))
          if int(tmp[0]) >= args.startb and tmp[1] == "CA":
            num2.append(int(tmp[2]))
            xC2.append(float(tmp[3]))
            yC2.append(float(tmp[4]))
            zC2.append(float(tmp[5]))
        except (ValueError, IndexError, AttributeError):
          i=0

a1 = []
a2 = []
dis = []
i = 0
while i < len(num):
  j = 0
  while j < len(num2):
    a1.append(num[i])
    a2.append(num2[j])
    dis.append(getdistance(xC[i],yC[i],zC[i],xC2[j],yC2[j],zC2[j]))
    j += 1
  i += 1

fa1 = []
fa2 = []
fdis = []
if len(num) < len(num2):
  i = 0
  while i < len(num):
    tempdis = 1000000000.0
    curlow = -1
    j = 0
    while j < len(dis):
      if dis[j] <= tempdis:
        tempdis = dis[j]
        curlow = j
      j += 1
    if (dis[curlow]+1.0) <= (args.boxr*0.45):
      fa1.append(a1[curlow])
      fa2.append(a2[curlow])
      fdis.append(dis[curlow])
    j = 0
    delitems = []
    while j < len(dis):
      breakb = 0
      if a1[j] == a1[curlow]:
        delitems.append(j)
        breakb = 1
      if a2[j] == a2[curlow] and breakb == 0:
        delitems.append(j)
      j += 1
    j = 0
    while j < len(delitems):
      delete = delitems[j] - j
      del a1[delete]
      del a2[delete]
      del dis[delete]
      j += 1
    i += 1
else:
  i = 0
  while i < len(num2):
    tempdis = 1000000000.0
    curlow = -1
    j = 0
    while j < len(dis):
      if dis[j] <= tempdis:
        tempdis = dis[j]
        curlow = j
      j += 1
    if (dis[curlow]+1.0) <= (args.boxr*0.45):
      fa1.append(a1[curlow])
      fa2.append(a2[curlow])
      fdis.append(dis[curlow])
    j = 0
    delitems = []
    while j < len(dis):
      breakb = 0
      if a1[j] == a1[curlow]:
        delitems.append(j)
        breakb = 1
      if a2[j] == a2[curlow] and breakb == 0:
        delitems.append(j)
      j += 1
    j = 0
    while j < len(delitems):
      delete = delitems[j] - j
      del a1[delete]
      del a2[delete]
      del dis[delete]
      j += 1
    i += 1

index = open("index.ndx", "a")
i = 0
while i < len(fdis):
  index.write("[ a_" + str(fa1[i]) + " ]" + "\n")
  index.write(str(fa1[i]) + "\n")
  index.write("[ a_" + str(fa2[i]) + " ]" + "\n")
  index.write(str(fa2[i]) + "\n")
  i += 1
index.close()

index = open("index_vac.ndx", "a")
i = 0
while i < len(fdis):
  index.write("[ a_" + str(fa1[i]) + " ]" + "\n")
  index.write(str(fa1[i]) + "\n")
  index.write("[ a_" + str(fa2[i]) + " ]" + "\n")
  index.write(str(fa2[i]) + "\n")
  i += 1
index.close()

k = 25000.0/len(fdis)
f = open("bind.mdp", "a")
f.write("pull-ngroups            = %.0f\n"%(len(fdis)*2))
f.write("pull-ncoords            = %.0f\n"%(len(fdis)))
f.write("\n")
j = 0
groupcount = 1
coordcount = 1
while j < len(fdis):
  f.write("pull-group%.0f-name        = a_%.0f\n"%(groupcount,fa1[j]))
  g1 = groupcount
  groupcount += 1
  f.write("pull-group%.0f-name        = a_%.0f\n"%(groupcount,fa2[j]))
  g2 = groupcount
  groupcount += 1
  f.write("pull-coord%.0f-type        = umbrella\n"%(coordcount))
  f.write("pull-coord%.0f-geometry    = distance\n"%(coordcount))
  f.write("pull-coord%.0f-dim         = Y Y Y\n"%(coordcount))
  f.write("pull-coord%.0f-start       = no\n"%(coordcount))
  f.write("pull-coord%.0f-rate        = 0.0002\n"%(coordcount))
  f.write("pull-coord%.0f-groups      = %.0f %.0f\n"%(coordcount,g1,g2))
  f.write("pull-coord%.0f-init        = %.3f\n"%(coordcount,fdis[j]))
  f.write("pull-coord%.0f-k           = %.3f\n"%(coordcount,k))
  f.write("\n")
  coordcount += 1
  j += 1
f.close()
