#!/usr/bin/env python
#
# PLEASE REPORT BUGS, QUESTIONS AND COMMENTS TO JAN.PERTHOLD@BOKU.AC.AT
#

import string
import math
import array
import os, sys, glob, re, time, argparse
import numpy as np

#------------------------------------------------------

parser = argparse.ArgumentParser(description="Calculate maximum distance between 2 atoms plus safety margin to box wall (i.e. the box diameter).")
parser.add_argument('-f','--confile', type=str, default="conf.gro", required=True, help="GROMACS coordinate file.")
args=parser.parse_args()

#------------------------------------------------------

def getdistance(x1, y1, z1, x2, y2, z2):
  """ Gives the distance between two points in 3D space.
  """  
  return math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2))

#------------------------------------------------------

# read coordinate file
num = []
xC = []
yC = []
zC = []
if os.path.isfile(args.confile):
  with open(args.confile, "rb") as f:
    for line in f:
      if not line.strip().startswith("#"):
        tmp = line.split()
        try:
            num.append(int(tmp[2]))
            xC.append(float(tmp[3]))
            yC.append(float(tmp[4]))
            zC.append(float(tmp[5]))
        except (ValueError, IndexError):
          i=0

maxR = 0.0
i = 0
while i < len(num):
  j = i + 1
  while j < len(num):
    disTemp = getdistance(xC[i],yC[i],zC[i],xC[j],yC[j],zC[j])
    if disTemp > maxR:
      maxR = disTemp
    j+=1
  i+=1

print str(maxR+3.0)
